// InformixAdaptor.h
// Copyright (c) 1994, NeXT Software, Inc.  All rights reserved.
//

#import <EOAccess/EOAccess.h>

@class InformixContext;

#define dbNameKey @"dbName"
#define userNameKey @"userName"
#define passwordKey @"password"

// Name of the UserDefaults domain for the Informix Adaptor
#define EOF_INFORMIX_ADAPTOR @"EOFInformixAdaptor"

#define NlsLangKey @"NLS_LANG"

@interface InformixAdaptor:EOAdaptor
{
    unsigned short _connectedLogons;
}

- initWithName:(NSString *)name;

- (Class)defaultExpressionClass;
    // Return [InformixSQLExpression class];
- (Class)adaptorContextClass;
    // Return [InformixContext class];
- (Class)adaptorChannelClass;
    // Return [InformixChannel class];

- createAdaptorContext;
    // Returns a new InformixContext, or nil if a new context cannot be
    // created.  InformixAdaptors by default have no contexts at all.
    // The newly created context retains its adaptor.

- (NSString *)informixConnectionString;
    // Return the connection string for the first arg of db_connect().
- (NSString *)informixUserName;
    // Return the username in the connection dictionary.
- (NSString *)informixPassword;
    // Return the password in the connection dictionary.
- (NSString *)informixDefaultValue:(NSString *)key;
    // Return the user default setting for the provided key.

- (BOOL)isValidQualifierType:(NSString *)typeName model:(EOModel *)model;
- (void)assertConnectionDictionaryIsValid;

- (void)informixContextWillConnect:(InformixContext *)logon;
- (void)informixContextDidDisconnect:(InformixContext *)logon;
    // Methods the InformixContext uses to notify the InformixAdaptor of
    // changes.

- (NSArray *)connectionKeys;
    // Standard keys for the connection dictionary.

- (void)raiseInformixError:(NSString *)sqlString;
    // Extract the error information in the connection structure and use
    // it to build and raise an exception.

+ (NSDictionary *)externalToInternalTypeMap;
    // This method returns a dictionary that maps each predefined externalType
    // known by the database to a default internal type.
+ (NSString *)internalTypeForExternalType:(NSString *)extType model:(EOModel *)model;
    // Return the interntal type ("NSString", "NSDecimalNumber," etc.) for
    // the provided external type ("VARCHAR", "DECIMAL", etc.).
+ (NSArray *)externalTypesWithModel:(EOModel *)model;
    // Return an array of available external type names
@end

