// InformixChannel.h 
// Copyright (c) 1994, NeXT Software, Inc.  All rights reserved.
//

#import "InformixDefines.h"
#import <EOAccess/EOAccess.h>

@class InformixContext;

@interface InformixChannel:EOAdaptorChannel
{
    struct informix_cursor _cursor;
    
    id _currentExpression;
    
    NSArray *_selectedAttributes;
    NSMutableArray *_selectedColumns;
    unsigned _fetchBufferLength;
    unsigned _rowCapacity;
    unsigned _rowsInBuffer;
    unsigned _nextRowInBuffer;
    unsigned _rowsProcessedCount;
    
    BOOL _fetchInProgress;
    BOOL _opened;
    BOOL _beganTransaction;

    long _connection;
}

- initWithInformixContext:(InformixContext *)logon;

- (struct informix_cursor *)cursorDataArea;
    // This returns NULL if the channel is not connected.

- (void)setFetchBufferLength:(unsigned)length;
- (unsigned)fetchBufferLength;
    // These are to manipulate how much memory to use as a fetch buffer.  The
    // larger the buffer, the more rows that can be returned for each round
    // trip to the server.

- (void)informixSetIsolationTo:(InformixIsolationLevel)isol;
    // Used for configuring the isolation transaction characteristics of this
    // connection.

@end


@interface NSObject (InformixChannelDelegation)

- (BOOL)informixChannel:(InformixChannel *)channel
    willReportDatabaseError:(NSString *)error;
    // This delegate message is called whenever the channel encounters an
    // error reported by the Informix 5.00 server.  The ... can be
    // gotten from the channel and context to determine exactly what error has
    // occurred.  The string error is the text of the error message which will
    // be sent to the adaptor's reportError: method.  The delegate can return
    // NO to prevent the channel from calling reportError:.

@end

