// OracleDescription.h 
// Copyright (c) 1994, NeXT Software, Inc.  All rights reserved.

#import <EOAccess/EOAccess.h>
#import "OracleChannel.h"

// This category on the OracleChannel implements the methods involved in
// reading an EOModel from the database.

@interface OracleChannel (ModelDescription)

+ (NSString *)oracleTableNamesSQL;
+ (void)setOracleTableNamesSQL:(NSString *)sql;
    // By default, the Oracle 7 adaptor evaluates the sql:
    //
    // SELECT TABLE_NAME FROM USER_TABLES ORDER BY TABLE_NAME
    //
    // to read a default list of table names in from the database.  If you
    // want a different set of default tables you can set a sql statement here
    // which will be used by all OracleChannels in the application, or you can
    // set a default sql statement through the dwrite:
    //
    // % defaults write Oracle7Adaptor EOOracleTableNamesSQL "SELECT TABLE_NAME FROM..."
    //
    // The only constraint on the sql is that the there be one item in the
    // select list, and that it selects the set of table names.

- (NSArray *)describeTableNames;
    // Returns an array of the names of all the tables owned by the current
    // user. Will use the SQL defined in EOOracleTableNamesSQL if it exists.
- (EOModel *)describeModelWithTableNames:(NSArray *)tableNames;
    // Constructs a default model out of the database's meta data.  It also
    // puts the adaptor name and connection dictionary in the new model.
    // This method may raise an exception if an error occurs.
- (void)describeStoredProceduresForModel:(EOModel *)model;
    // For every function and stored procedure found in the ALL_OBJECTS view,
    // the method will create a corresponding entry in the model's list of
    // stored procedures.

@end

